@@ -1,5 +1,3 @@
-from typing import Tuple, Type
-
 from redis import Connection as RedisConnection
 from redis import Redis
 
@@ -8,7 +6,7 @@ class NoRedisConnectionException(Exception):
     pass
 
 
-def parse_connection(connection: Redis) -> Tuple[Type[Redis], Type[RedisConnection], dict]:
+def parse_connection(connection: Redis) -> tuple[type[Redis], type[RedisConnection], dict]:
     connection_pool_kwargs = connection.connection_pool.connection_kwargs.copy()
     connection_pool_class = connection.connection_pool.connection_class
 
