@@ -4,9 +4,10 @@ import json
 import logging
 import warnings
 import zlib
+from collections.abc import Iterable, Mapping, Sequence
 from datetime import datetime, timedelta
 from enum import Enum
-from typing import TYPE_CHECKING, Any, Callable, Dict, Iterable, List, Mapping, Optional, Tuple, Type, Union
+from typing import TYPE_CHECKING, Any, Callable, Optional, Union, cast
 from uuid import uuid4
 
 from redis import WatchError
@@ -38,10 +39,10 @@ from .utils import (
     decode_redis_hash,
     ensure_job_list,
     get_call_string,
-    get_version,
     import_attribute,
     now,
     parse_timeout,
+    resolve_function_reference,
     str_to_date,
     utcformat,
 )
@@ -52,6 +53,7 @@ logger = logging.getLogger('rq.job')
 class JobStatus(str, Enum):
     """The Status of Job within its lifecycle at any given time."""
 
+    CREATED = 'created'
     QUEUED = 'queued'
     FINISHED = 'finished'
     FAILED = 'failed'
@@ -70,22 +72,24 @@ def parse_job_id(job_or_execution_id: str) -> str:
 
 
 class Dependency:
-    dependencies: List[Union['Job', str]]
+    dependencies: Sequence[Union['Job', str]]
 
-    def __init__(self, jobs: Iterable[Union['Job', str]], allow_failure: bool = False, enqueue_at_front: bool = False):
+    def __init__(
+        self,
+        jobs: Union['Job', str, Sequence[Union['Job', str]]],
+        allow_failure: bool = False,
+        enqueue_at_front: bool = False,
+    ):
         """The definition of a Dependency.
 
         Args:
-            jobs (Iterable[Union[Job, str]]): An iterable of Job instances or Job IDs.
+            jobs (Union[Job, str, Sequence[Union[Job, str]]]): A Job, Job ID, or sequence of Job instances/Job IDs.
                 Anything different will raise a ValueError
             allow_failure (bool, optional): Whether to allow for failure when running the dependency,
                 meaning, the dependencies should continue running even after one of them failed.
                 Defaults to False.
             enqueue_at_front (bool, optional): Whether this dependency should be enqueued at the front of the queue.
                 Defaults to False.
-
-        Raises:
-            ValueError: If the `jobs` param has anything different than `str` or `Job` class or the job list is empty
         """
         dependent_jobs = ensure_job_list(jobs)
         if not all(isinstance(job, Job) or isinstance(job, str) for job in dependent_jobs if job):
@@ -164,19 +168,19 @@ class Job:
         self.connection = connection
         self._id = id
         self.created_at = now()
-        self._data = UNEVALUATED
-        self._func_name: Union[str, 'UnevaluatedType'] = UNEVALUATED
-        self._instance: Optional[Union[object, 'UnevaluatedType']] = UNEVALUATED
-        self._args: Union[tuple, list, 'UnevaluatedType'] = UNEVALUATED
-        self._kwargs: Union[Dict[str, Any], 'UnevaluatedType'] = UNEVALUATED
+        self._data: Union[bytes, 'UnevaluatedType'] = UNEVALUATED
+        self._func_name: Union[str, UnevaluatedType] = UNEVALUATED
+        self._instance: Optional[Union[object, UnevaluatedType]] = UNEVALUATED
+        self._args: Union[tuple, list, UnevaluatedType] = UNEVALUATED
+        self._kwargs: Union[dict[str, Any], UnevaluatedType] = UNEVALUATED
         self._success_callback_name: Optional[str] = None
-        self._success_callback: Union[Callable[['Job', 'Redis', Any], Any], 'UnevaluatedType'] = UNEVALUATED
+        self._success_callback: Union[Callable[[Job, Redis, Any], Any], UnevaluatedType] = UNEVALUATED
         self._failure_callback_name: Optional[str] = None
-        self._failure_callback: Union[Callable[['Job', 'Redis', Unpack[Tuple['ExcInfo']]], Any], 'UnevaluatedType'] = (
+        self._failure_callback: Union[Callable[[Job, Redis, Unpack[tuple[ExcInfo]]], Any], UnevaluatedType] = (
             UNEVALUATED
         )
         self._stopped_callback_name: Optional[str] = None
-        self._stopped_callback: Union[Callable[['Job', 'Redis'], Any], 'UnevaluatedType'] = UNEVALUATED
+        self._stopped_callback: Union[Callable[[Job, Redis], Any], UnevaluatedType, None] = UNEVALUATED
         self.description: Optional[str] = None
         self.origin: str = ''
         self.enqueued_at: Optional[datetime] = None
@@ -192,32 +196,33 @@ class Job:
         self.failure_ttl: Optional[int] = None
         self.ttl: Optional[int] = None
         self.worker_name: Optional[str] = None
-        self._status: Optional[JobStatus] = None
-        self._dependency_ids: List[str] = []
-        self.meta: Dict[str, Any] = {}
+        self._status: JobStatus = JobStatus.CREATED
+        self._dependency_ids: list[str] = []
+        self.meta: dict[str, Any] = {}
         self.serializer = resolve_serializer(serializer)
         self.retries_left: Optional[int] = None
         self.number_of_retries: Optional[int] = None
-        self.retry_intervals: Optional[List[int]] = None
-        self.redis_server_version: Optional[Tuple[int, int, int]] = None
+        self.retry_intervals: Optional[list[int]] = None
+        self.redis_server_version: Optional[tuple[int, int, int]] = None
         self.last_heartbeat: Optional[datetime] = None
         self.allow_dependency_failures: Optional[bool] = None
         self.enqueue_at_front: Optional[bool] = None
         self.group_id: Optional[str] = None
 
         self.repeats_left: Optional[int] = None
-        self.repeat_intervals: Optional[List[int]] = None
+        self.repeat_intervals: Optional[list[int]] = None
 
         from .results import Result
 
         self._cached_result: Optional[Result] = None
+        self.log = logger
 
     @classmethod
     def create(
         cls,
         func: FunctionReferenceType,
         args: Optional[Union[list, tuple]] = None,
-        kwargs: Optional[Dict[str, Any]] = None,
+        kwargs: Optional[dict[str, Any]] = None,
         connection: Optional['Redis'] = None,
         result_ttl: Optional[int] = None,
         ttl: Optional[int] = None,
@@ -227,7 +232,7 @@ class Job:
         timeout: Optional[int] = None,
         id: Optional[str] = None,
         origin: str = '',
-        meta: Optional[Dict[str, Any]] = None,
+        meta: Optional[dict[str, Any]] = None,
         failure_ttl: Optional[int] = None,
         serializer=None,
         group_id: Optional[str] = None,
@@ -291,11 +296,6 @@ class Job:
         if kwargs is None:
             kwargs = {}
 
-        if not isinstance(args, (tuple, list)):
-            raise TypeError('{0!r} is not a valid args list'.format(args))
-        if not isinstance(kwargs, dict):
-            raise TypeError('{0!r} is not a valid kwargs dict'.format(kwargs))
-
         job = cls(connection=connection, serializer=serializer)
         if id is not None:
             job.set_id(id)
@@ -304,19 +304,7 @@ class Job:
             job.origin = origin
 
         # Set the core job tuple properties
-        job._instance = None
-        if inspect.ismethod(func):
-            job._instance = func.__self__
-            job._func_name = func.__name__
-        elif inspect.isfunction(func) or inspect.isbuiltin(func):
-            job._func_name = '{0}.{1}'.format(func.__module__, func.__qualname__)
-        elif isinstance(func, str):
-            job._func_name = as_text(func)
-        elif not inspect.isclass(func) and hasattr(func, '__call__'):  # a callable class instance
-            job._instance = func
-            job._func_name = '__call__'
-        else:
-            raise TypeError('Expected a callable or a string, but got: {0}'.format(func))
+        job._instance, job._func_name = resolve_function_reference(func)
         job._args = args
         job._kwargs = kwargs
 
@@ -356,23 +344,18 @@ class Job:
         job.failure_ttl = parse_timeout(failure_ttl)
         job.ttl = parse_timeout(ttl)
         job.timeout = parse_timeout(timeout)
-        job._status = status
         job.meta = meta or {}
         job.group_id = group_id
 
-        # dependency could be job instance or id, or iterable thereof
+        # Process job dependencies
         if depends_on is not None:
-            depends_on_list = []
-            for depends_on_item in ensure_job_list(depends_on):
-                if isinstance(depends_on_item, Dependency):
-                    # If a Dependency has enqueue_at_front or allow_failure set to True, these behaviors are used for
-                    # all dependencies.
-                    job.enqueue_at_front = job.enqueue_at_front or depends_on_item.enqueue_at_front
-                    job.allow_dependency_failures = job.allow_dependency_failures or depends_on_item.allow_failure
-                    depends_on_list.extend(depends_on_item.dependencies)
-                else:
-                    depends_on_list.extend(ensure_job_list(depends_on_item))
-            job._dependency_ids = [dep.id if isinstance(dep, Job) else dep for dep in depends_on_list]
+            job.process_dependencies(depends_on)
+
+        # Set status: explicit status takes precedence, otherwise DEFERRED if has dependencies, CREATED if not
+        if status is not None:
+            job._status = status
+        else:
+            job._status = JobStatus.CREATED
 
         return job
 
@@ -389,7 +372,7 @@ class Job:
             return q.get_job_position(self.id)
         return None
 
-    def get_status(self, refresh: bool = True) -> Optional[JobStatus]:
+    def get_status(self, refresh: bool = True) -> JobStatus:
         """Gets the Job Status
 
         Args:
@@ -416,10 +399,10 @@ class Job:
             pipeline (Optional[Pipeline], optional): Optional Redis Pipeline to use. Defaults to None.
         """
         self._status = status
-        connection: 'Redis' = pipeline if pipeline is not None else self.connection
+        connection: Redis = pipeline if pipeline is not None else self.connection
         connection.hset(self.key, 'status', self._status)
 
-    def get_meta(self, refresh: bool = True) -> Dict:
+    def get_meta(self, refresh: bool = True) -> dict:
         """Get's the metadata for a Job, an arbitrary dictionary.
 
         Args:
@@ -430,7 +413,7 @@ class Job:
         """
         if refresh:
             meta = self.connection.hget(self.key, 'meta')
-            self.meta = self.serializer.loads(meta) if meta else {}  # type: ignore[assignment]
+            self.meta = self.serializer.loads(meta) if meta else {}
 
         return self.meta
 
@@ -488,7 +471,7 @@ class Job:
         return job
 
     @property
-    def dependent_ids(self) -> List[str]:
+    def dependent_ids(self) -> list[str]:
         """Returns a list of ids of jobs whose execution depends on this
         job's successful execution."""
         return list(map(as_text, self.connection.smembers(self.dependents_key)))
@@ -502,7 +485,7 @@ class Job:
         if self.instance:
             return getattr(self.instance, func_name)
 
-        return import_attribute(self.func_name)
+        return import_attribute(func_name)
 
     @property
     def success_callback(self) -> Optional[SuccessCallbackType]:
@@ -539,14 +522,15 @@ class Job:
         return self._failure_callback_timeout
 
     @property
-    def stopped_callback(self):
+    def stopped_callback(self) -> Optional[Callable[['Job', 'Redis'], Any]]:
         if self._stopped_callback is UNEVALUATED:
             if self._stopped_callback_name:
                 self._stopped_callback = import_attribute(self._stopped_callback_name)
             else:
                 self._stopped_callback = None
 
-        return self._stopped_callback
+        # After deserialization, _stopped_callback is either a callable or None, never UNEVALUATED
+        return cast(Optional[Callable[['Job', 'Redis'], Any]], self._stopped_callback)
 
     @property
     def stopped_callback_timeout(self) -> int:
@@ -568,7 +552,7 @@ class Job:
             raise DeserializationError() from e
 
     @property
-    def data(self):
+    def data(self) -> bytes:
         if self._data is UNEVALUATED:
             if self._func_name is UNEVALUATED:
                 raise ValueError('Cannot build the job data')
@@ -584,7 +568,7 @@ class Job:
 
             job_tuple = self._func_name, self._instance, self._args, self._kwargs
             self._data = self.serializer.dumps(job_tuple)
-        return self._data
+        return cast(bytes, self._data)
 
     @data.setter
     def data(self, value):
@@ -595,10 +579,10 @@ class Job:
         self._kwargs = UNEVALUATED
 
     @property
-    def func_name(self):
+    def func_name(self) -> Optional[str]:
         if self._func_name is UNEVALUATED:
             self._deserialize_data()
-        return self._func_name
+        return self._func_name  # type: ignore[return-value]
 
     @func_name.setter
     def func_name(self, value):
@@ -628,7 +612,7 @@ class Job:
         self._data = UNEVALUATED
 
     @property
-    def kwargs(self) -> Dict[str, Any]:
+    def kwargs(self) -> dict[str, Any]:
         if self._kwargs is UNEVALUATED:
             self._deserialize_data()
         return self._kwargs  # type: ignore[return-value]
@@ -671,7 +655,7 @@ class Job:
         return job
 
     @classmethod
-    def fetch_many(cls, job_ids: Iterable[str], connection: 'Redis', serializer=None) -> List[Optional['Job']]:
+    def fetch_many(cls, job_ids: Iterable[str], connection: 'Redis', serializer=None) -> list[Optional['Job']]:
         """
         Bulk version of Job.fetch
 
@@ -692,7 +676,7 @@ class Job:
                 pipeline.hgetall(cls.key_for(job_id))
             results = pipeline.execute()
 
-        jobs: List[Optional['Job']] = []
+        jobs: list[Optional[Job]] = []
         for i, job_id in enumerate(parsed_ids):
             if not results[i]:
                 jobs.append(None)
@@ -705,10 +689,10 @@ class Job:
         return jobs
 
     def __repr__(self):  # noqa  # pragma: no cover
-        return '{0}({1!r}, enqueued_at={2!r})'.format(self.__class__.__name__, self._id, self.enqueued_at)
+        return f'{self.__class__.__name__}({self._id!r}, enqueued_at={self.enqueued_at!r})'
 
     def __str__(self):
-        return '<{0} {1}: {2}>'.format(self.__class__.__name__, self.id, self.description)
+        return f'<{self.__class__.__name__} {self.id}: {self.description}>'
 
     def __eq__(self, other):  # noqa
         return isinstance(other, self.__class__) and self.id == other.id
@@ -735,7 +719,7 @@ class Job:
             value (str): The value to set as Job ID
         """
         if not isinstance(value, str):
-            raise TypeError('id must be a string, not {0}'.format(type(value)))
+            raise TypeError(f'id must be a string, not {type(value)}')
 
         if ':' in value:
             raise ValueError('id must not contain ":"')
@@ -761,16 +745,16 @@ class Job:
     id = property(get_id, set_id)
 
     @classmethod
-    def key_for(cls, job_id: str) -> bytes:
+    def key_for(cls, job_id: str) -> str:
         """The Redis key that is used to store job hash under.
 
         Args:
             job_id (str): The Job ID
 
         Returns:
-            redis_job_key (bytes): The Redis fully qualified key for the job
+            redis_job_key (str): The Redis fully qualified key for the job
         """
-        return (cls.redis_job_namespace_prefix + job_id).encode('utf-8')
+        return cls.redis_job_namespace_prefix + job_id
 
     @classmethod
     def dependents_key_for(cls, job_id: str) -> str:
@@ -782,7 +766,7 @@ class Job:
         Returns:
             dependents_key (str): The dependents key
         """
-        return '{0}{1}:dependents'.format(cls.redis_job_namespace_prefix, job_id)
+        return f'{cls.redis_job_namespace_prefix}{job_id}:dependents'
 
     @property
     def key(self):
@@ -796,9 +780,9 @@ class Job:
 
     @property
     def dependencies_key(self):
-        return '{0}:{1}:dependencies'.format(self.redis_job_namespace_prefix, self.id)
+        return f'{self.redis_job_namespace_prefix}:{self.id}:dependencies'
 
-    def fetch_dependencies(self, watch: bool = False, pipeline: Optional['Pipeline'] = None) -> List['Job']:
+    def fetch_dependencies(self, watch: bool = False, pipeline: Optional['Pipeline'] = None) -> list['Job']:
         """Fetch all of a job's dependencies. If a pipeline is supplied, and
         watch is true, then set WATCH on all the keys of all dependencies.
 
@@ -833,12 +817,11 @@ class Job:
 
         from .results import Result
 
-        if self.supports_redis_streams:
-            if not self._cached_result:
-                self._cached_result = self.latest_result()
+        if not self._cached_result:
+            self._cached_result = self.latest_result()
 
-            if self._cached_result and self._cached_result.type == Result.Type.FAILED:
-                return self._cached_result.exc_string
+        if self._cached_result and self._cached_result.type == Result.Type.FAILED:
+            return self._cached_result.exc_string
 
         return self._exc_info
 
@@ -856,17 +839,6 @@ class Job:
         if refresh:
             self._cached_result = None
 
-        if not self.supports_redis_streams:
-            if self._result is not None:
-                return self._result
-
-            rv = self.connection.hget(self.key, 'result')
-            if rv is not None:
-                # cache the result
-                self._result = self.serializer.loads(rv)
-                return self._result
-            return None
-
         if not self._cached_result:
             self._cached_result = self.latest_result()
 
@@ -897,13 +869,13 @@ class Job:
 
         from .results import Result
 
-        if self.supports_redis_streams:
-            if not self._cached_result:
-                self._cached_result = self.latest_result()
+        if not self._cached_result:
+            self._cached_result = self.latest_result()
 
-            if self._cached_result and self._cached_result.type == Result.Type.SUCCESSFUL:
-                return self._cached_result.return_value
+        if self._cached_result and self._cached_result.type == Result.Type.SUCCESSFUL:
+            return self._cached_result.return_value
 
+        # TODO: Remove this fallback in RQ 3.0 - only kept for backward compatibility
         # Fallback to old behavior of getting result from job hash
         if self._result is None:
             rv = self.connection.hget(self.key, 'result')
@@ -912,7 +884,7 @@ class Job:
                 self._result = self.serializer.loads(rv)
         return self._result
 
-    def results(self) -> List['Result']:
+    def results(self) -> list['Result']:
         """Returns all Result objects
 
         Returns:
@@ -948,7 +920,7 @@ class Job:
         try:
             raw_data = obj['data']
         except KeyError:
-            raise NoSuchJobError('Unexpected job format: {0}'.format(obj))
+            raise NoSuchJobError(f'Unexpected job format: {obj}')
 
         try:
             self.data = zlib.decompress(raw_data)
@@ -956,14 +928,16 @@ class Job:
             # Fallback to uncompressed string
             self.data = raw_data
 
-        self.created_at = str_to_date(obj.get('created_at'))  # type: ignore[assignment]
+        self.created_at = str_to_date(v) if (v := obj.get('created_at')) else now()
         self.origin = as_text(obj['origin']) if obj.get('origin') else ''
         self.worker_name = obj['worker_name'].decode() if obj.get('worker_name') else None
         self.description = as_text(obj['description']) if obj.get('description') else None
-        self.enqueued_at = str_to_date(obj.get('enqueued_at'))
-        self.started_at = str_to_date(obj.get('started_at'))
-        self.ended_at = str_to_date(obj.get('ended_at'))
-        self.last_heartbeat = str_to_date(obj.get('last_heartbeat'))
+
+        self.enqueued_at = str_to_date(v) if (v := obj.get('enqueued_at')) else None
+        self.started_at = str_to_date(v) if (v := obj.get('started_at')) else None
+        self.ended_at = str_to_date(v) if (v := obj.get('ended_at')) else None
+
+        self.last_heartbeat = str_to_date(v) if (v := obj.get('last_heartbeat')) else None
         self.group_id = as_text(obj['group_id']) if obj.get('group_id') else None
         result = obj.get('result')
         if result:
@@ -974,7 +948,11 @@ class Job:
         self.timeout = parse_timeout(obj.get('timeout')) if obj.get('timeout') else None
         self.result_ttl = int(obj['result_ttl']) if obj.get('result_ttl') else None
         self.failure_ttl = int(obj['failure_ttl']) if obj.get('failure_ttl') else None
-        self._status = JobStatus(as_text(obj['status'])) if obj.get('status') else None
+
+        # Beginning from v2.4.1, jobs are created with a status, so the fallback to CREATED
+        # is not needed, but we keep it for backwards compatibility
+        # In future versions, if a job has no status, an error should be raised
+        self._status = JobStatus(as_text(obj['status'])) if obj.get('status') else JobStatus.CREATED
 
         if obj.get('success_callback_name'):
             self._success_callback_name = obj['success_callback_name'].decode()
@@ -1002,7 +980,7 @@ class Job:
         self.enqueue_at_front = bool(int(obj['enqueue_at_front'])) if 'enqueue_at_front' in obj else None
         self.ttl = int(obj['ttl']) if obj.get('ttl') else None
         try:
-            self.meta = self.serializer.loads(obj['meta']) if obj.get('meta') else {}  # type: ignore[assignment]
+            self.meta = self.serializer.loads(obj['meta']) if obj.get('meta') else {}
         except Exception:  # depends on the serializer
             self.meta = {'unserialized': obj.get('meta', {})}
 
@@ -1032,7 +1010,7 @@ class Job:
         """
         data = self.connection.hgetall(self.key)
         if not data:
-            raise NoSuchJobError('No such job: {0}'.format(self.key))
+            raise NoSuchJobError(f'No such job: {self.key}')
         self.restore(data)
 
     def to_dict(self, include_meta: bool = True, include_result: bool = True) -> dict:
@@ -1048,7 +1026,7 @@ class Job:
         Returns:
             dict: The Job serialized as a dictionary
         """
-        obj: Dict[str, Any] = {
+        obj: dict[str, Any] = {
             'created_at': utcformat(self.created_at or now()),
             'data': zlib.compress(self.data),
             'success_callback_name': self._success_callback_name if self._success_callback_name else '',
@@ -1099,8 +1077,7 @@ class Job:
             obj['result_ttl'] = self.result_ttl
         if self.failure_ttl is not None:
             obj['failure_ttl'] = self.failure_ttl
-        if self._status is not None:
-            obj['status'] = self._status
+        obj['status'] = self._status
         if self._dependency_ids:
             obj['dependency_id'] = self._dependency_ids[0]  # for backwards compatibility
             obj['dependency_ids'] = json.dumps(self._dependency_ids)
@@ -1118,6 +1095,7 @@ class Job:
 
         return obj
 
+    # TODO: Remove include_result parameter in RQ 3.0 - results are now always saved to Redis streams
     def save(self, pipeline: Optional['Pipeline'] = None, include_meta: bool = True, include_result: bool = True):
         """Dumps the current job instance to its corresponding Redis key.
 
@@ -1131,6 +1109,7 @@ class Job:
             pipeline (Optional[Pipeline], optional): The Redis' pipeline to use. Defaults to None.
             include_meta (bool, optional): Whether to include the job's metadata. Defaults to True.
             include_result (bool, optional): Whether to include the job's result. Defaults to True.
+                TODO: Remove this parameter in RQ 3.0 - results are now always saved to Redis streams.
         """
         key = self.key
         connection = pipeline if pipeline is not None else self.connection
@@ -1138,28 +1117,17 @@ class Job:
         mapping = self.to_dict(include_meta=include_meta, include_result=include_result)
         connection.hset(key, mapping=mapping)
 
-    @property
-    def supports_redis_streams(self) -> bool:
-        """Only supported by Redis server >= 5.0 is required."""
-        return self.get_redis_server_version() >= (5, 0, 0)
-
-    def get_redis_server_version(self) -> Tuple[int, int, int]:
-        """Return Redis server version of connection
-
-        Returns:
-            redis_server_version (Tuple[int, int, int]): The Redis version within a Tuple of integers, eg (5, 0, 9)
-        """
-        if self.redis_server_version is None:
-            self.redis_server_version = get_version(self.connection)
-
-        return self.redis_server_version
-
     def save_meta(self):
         """Stores job meta from the job instance to the corresponding Redis key."""
         meta = self.serializer.dumps(self.meta)
         self.connection.hset(self.key, 'meta', meta)
 
-    def cancel(self, pipeline: Optional['Pipeline'] = None, enqueue_dependents: bool = False):
+    def cancel(
+        self,
+        pipeline: Optional['Pipeline'] = None,
+        enqueue_dependents: bool = False,
+        remove_from_dependencies: bool = False,
+    ):
         """Cancels the given job, which will prevent the job from ever being
         ran (or inspected).
 
@@ -1178,7 +1146,7 @@ class Job:
             InvalidJobOperation: If the job has already been cancelled.
         """
         if self.is_canceled:
-            raise InvalidJobOperation('Cannot cancel already canceled job: {}'.format(self.get_id()))
+            raise InvalidJobOperation(f'Cannot cancel already canceled job: {self.get_id()}')
         from .queue import Queue
         from .registry import CanceledJobRegistry
 
@@ -1196,6 +1164,12 @@ class Job:
                     if pipeline is None:
                         pipe.watch(self.dependents_key)
                     q.enqueue_dependents(self, pipeline=pipeline, exclude_job_id=self.id)
+
+                if remove_from_dependencies:
+                    # Go through all dependencies and remove the current job from each dependency's dependents_key
+                    for dependency in self.fetch_dependencies(pipeline=pipe):
+                        pipe.srem(dependency.dependents_key, self.id)
+
                 self._remove_from_registries(pipeline=pipe, remove_from_queue=True)
 
                 registry = CanceledJobRegistry(
@@ -1231,7 +1205,7 @@ class Job:
 
         return ExecutionRegistry(self.id, connection=self.connection)
 
-    def get_executions(self) -> List['Execution']:
+    def get_executions(self) -> list['Execution']:
         return self.execution_registry.get_executions()
 
     def _remove_from_registries(self, pipeline: Optional['Pipeline'] = None, remove_from_queue: bool = True):
@@ -1346,7 +1320,29 @@ class Job:
             assert self is _job_stack.pop()
         return self._result
 
-    def prepare_for_execution(self, worker_name: str, pipeline: 'Pipeline'):
+    def process_dependencies(self, depends_on: JobDependencyType) -> None:
+        """Process job dependencies and set dependency-related attributes.
+
+        Args:
+            depends_on: Job dependencies - can be a Dependency, Job, string ID,
+                       or iterable of these types.
+        """
+
+        depends_on_list: list[Union['Job', str]] = []
+        for depends_on_item in ensure_job_list(depends_on):
+            if isinstance(depends_on_item, Dependency):
+                # If a Dependency has enqueue_at_front or allow_failure set to True, these behaviors are used for
+                # all dependencies.
+                self.enqueue_at_front = self.enqueue_at_front or depends_on_item.enqueue_at_front
+                self.allow_dependency_failures = self.allow_dependency_failures or depends_on_item.allow_failure
+                depends_on_list.extend(list(depends_on_item.dependencies))
+            else:
+                # After checking for Dependency, depends_on_item should be Job or str
+                # Use type cast to inform mypy of the narrowed type
+                depends_on_list.append(depends_on_item)  # type: ignore[arg-type]
+        self._dependency_ids = [dep.id if isinstance(dep, Job) else dep for dep in depends_on_list]
+
+    def prepare_for_execution(self, worker_name: str, pipeline: 'Pipeline') -> None:
         """Prepares the job for execution, setting the worker name,
         heartbeat information, status and other metadata before execution begins.
 
@@ -1375,6 +1371,8 @@ class Job:
         Returns:
             result (Any): The function result
         """
+        if not self.func:
+            raise ValueError('Cannot execute job: function is None')
         result = self.func(*self.args, **self.kwargs)
         if asyncio.iscoroutine(result):
             loop = asyncio.new_event_loop()
@@ -1466,7 +1464,7 @@ class Job:
             self.origin, connection=self.connection, job_class=self.__class__, serializer=self.serializer
         )
 
-    def execute_success_callback(self, death_penalty_class: Type[BaseDeathPenalty], result: Any):
+    def execute_success_callback(self, death_penalty_class: type[BaseDeathPenalty], result: Any):
         """Executes success_callback for a job.
         with timeout .
 
@@ -1477,76 +1475,78 @@ class Job:
         if not self.success_callback:
             return
 
-        logger.debug('Running success callbacks for %s', self.id)
+        self.log.debug('Job %s: running success callback...', self.id)
         with death_penalty_class(self.success_callback_timeout, JobTimeoutException, job_id=self.id):
             self.success_callback(self, self.connection, result)
 
-    def execute_failure_callback(self, death_penalty_class: Type[BaseDeathPenalty], *exc_info):
+    def execute_failure_callback(self, death_penalty_class: type[BaseDeathPenalty], *exc_info):
         """Executes failure_callback with possible timeout"""
         if not self.failure_callback:
             return
 
-        logger.debug('Running failure callbacks for %s', self.id)
+        self.log.debug('Job %s: running failure callback...', self.id)
         try:
             with death_penalty_class(self.failure_callback_timeout, JobTimeoutException, job_id=self.id):
                 self.failure_callback(self, self.connection, *exc_info)
         except Exception:  # noqa
-            logger.exception('Job %s: error while executing failure callback', self.id)
+            self.log.exception('Job %s: error while executing failure callback', self.id)
             raise
 
-    def execute_stopped_callback(self, death_penalty_class: Type[BaseDeathPenalty]):
+    def execute_stopped_callback(self, death_penalty_class: type[BaseDeathPenalty]):
         """Executes stopped_callback with possible timeout"""
-        logger.debug('Running stopped callbacks for %s', self.id)
+        if self.stopped_callback is None:
+            return
+
+        self.log.debug('Job %s: running stopped callback...', self.id)
         try:
             with death_penalty_class(self.stopped_callback_timeout, JobTimeoutException, job_id=self.id):
                 self.stopped_callback(self, self.connection)
         except Exception:  # noqa
-            logger.exception('Job %s: error while executing stopped callback', self.id)
+            self.log.exception('Job %s: error while executing stopped callback', self.id)
             raise
 
-    def _handle_success(self, result_ttl: int, pipeline: 'Pipeline'):
+    def _handle_success(self, result_ttl, pipeline: 'Pipeline', worker_name: str = ''):
         """Saves and cleanup job after successful execution"""
-        # self.log.debug('Setting job %s status to finished', job.id)
-        self.set_status(JobStatus.FINISHED, pipeline=pipeline)
-        # Result should be saved in job hash only if server
-        # doesn't support Redis streams
-        include_result = not self.supports_redis_streams
-        # Don't clobber user's meta dictionary!
-        self.save(pipeline=pipeline, include_meta=False, include_result=include_result)
-        # Result creation should eventually be moved to job.save() after support
-        # for Redis < 5.0 is dropped. job.save(include_result=...) is used to test
-        # for backward compatibility
-        if self.supports_redis_streams:
-            from .results import Result
+        self.log.debug('Job %s: handling success...', self.id)
 
-            Result.create(self, Result.Type.SUCCESSFUL, return_value=self._result, ttl=result_ttl, pipeline=pipeline)
+        self.set_status(JobStatus.FINISHED, pipeline=pipeline)
+        # Don't clobber user's meta dictionary!
+        self.save(pipeline=pipeline, include_meta=False, include_result=False)
+        from .results import Result
+
+        Result.create(
+            self,
+            Result.Type.SUCCESSFUL,
+            return_value=self._result,
+            ttl=result_ttl,
+            worker_name=worker_name,
+            pipeline=pipeline,
+        )
 
         if result_ttl != 0:
             finished_job_registry = self.finished_job_registry
             finished_job_registry.add(self, result_ttl, pipeline)
 
-    def _handle_failure(self, exc_string: str, pipeline: 'Pipeline'):
+    def _handle_failure(self, exc_string: str, pipeline: 'Pipeline', worker_name: str = ''):
+        self.log.debug(
+            'Job %s: handling failure: %s', self.id, exc_string[:200] + '...' if len(exc_string) > 200 else exc_string
+        )
+
         failed_job_registry = self.failed_job_registry
-        # Exception should be saved in job hash if server
-        # doesn't support Redis streams
-        _save_exc_to_job = not self.supports_redis_streams
         failed_job_registry.add(
             self,
             ttl=self.failure_ttl,
             exc_string=exc_string,
             pipeline=pipeline,
-            _save_exc_to_job=_save_exc_to_job,
         )
-        if self.supports_redis_streams:
-            from .results import Result
+        from .results import Result
 
-            Result.create_failure(self, self.failure_ttl, exc_string=exc_string, pipeline=pipeline)
+        Result.create_failure(self, self.failure_ttl, exc_string=exc_string, worker_name=worker_name, pipeline=pipeline)
 
-    def _handle_retry_result(self, queue: 'Queue', pipeline: 'Pipeline'):
-        if self.supports_redis_streams:
-            from .results import Result
+    def _handle_retry_result(self, queue: 'Queue', pipeline: 'Pipeline', worker_name: str = ''):
+        from .results import Result
 
-            Result.create_retried(self, self.failure_ttl, pipeline=pipeline)
+        Result.create_retried(self, self.failure_ttl, worker_name=worker_name, pipeline=pipeline)
         self.number_of_retries = 1 if not self.number_of_retries else self.number_of_retries + 1
         queue._enqueue_job(self, pipeline=pipeline)
 
@@ -1586,12 +1586,12 @@ class Job:
             scheduled_datetime = now() + timedelta(seconds=retry_interval)
             self.set_status(JobStatus.SCHEDULED)
             queue.schedule_job(self, scheduled_datetime, pipeline=pipeline)
-            logger.info(
+            self.log.info(
                 'Job %s: scheduled for retry at %s, %s remaining', self.id, scheduled_datetime, self.retries_left
             )
         else:
             queue._enqueue_job(self, pipeline=pipeline)
-            logger.info('Job %s: enqueued for retry, %s remaining', self.id, self.retries_left)
+            self.log.info('Job %s: enqueued for retry, %s remaining', self.id, self.retries_left)
 
     def register_dependency(self, pipeline: Optional['Pipeline'] = None):
         """Jobs may have dependencies. Jobs are enqueued only if the jobs they
@@ -1610,6 +1610,8 @@ class Job:
         """
         from .registry import DeferredJobRegistry
 
+        self.log.debug('Job %s registering dependencies: %s', self.id, self._dependency_ids)
+
         registry = DeferredJobRegistry(
             self.origin, connection=self.connection, job_class=self.__class__, serializer=self.serializer
         )
@@ -1623,9 +1625,9 @@ class Job:
             connection.sadd(self.dependencies_key, dependency_id)
 
     @property
-    def dependency_ids(self) -> List[bytes]:
+    def dependency_ids(self) -> list[str]:
         dependencies = self.connection.smembers(self.dependencies_key)
-        return [Job.key_for(_id.decode()) for _id in dependencies]
+        return [_id.decode() for _id in dependencies]
 
     def dependencies_are_met(
         self,
@@ -1669,9 +1671,18 @@ class Job:
             # If parent job is not finished, we should only continue
             # if this job allows parent job to fail
             dependencies_ids.discard(parent_job.id)
-            if parent_job.get_status(refresh=refresh_job_status) == JobStatus.CANCELED:
+            parent_status = parent_job.get_status(refresh=refresh_job_status)
+            self.log.debug(
+                'Job %s parent job %s status: %s, allow_dependency_failures: %s',
+                self.id,
+                parent_job.id,
+                parent_status,
+                self.allow_dependency_failures,
+            )
+
+            if parent_status == JobStatus.CANCELED:
                 return False
-            elif parent_job.get_status(refresh=False) == JobStatus.FAILED and not self.allow_dependency_failures:
+            elif parent_status == JobStatus.FAILED and not self.allow_dependency_failures:
                 return False
 
             # If the only dependency is parent job, dependency has been met
@@ -1725,7 +1736,7 @@ class Retry:
         self.intervals = intervals
 
     @classmethod
-    def get_interval(cls, count: int, intervals: Union[int, List[int], None]) -> int:
+    def get_interval(cls, count: int, intervals: Union[int, list[int], None]) -> int:
         """Returns the appropriate retry interval based on retry count and intervals.
         If intervals is an integer, returns that value directly.
         If intervals is a list and retry count is bigger than length of intervals,
@@ -1764,4 +1775,5 @@ class Callback:
     def name(self) -> str:
         if isinstance(self.func, str):
             return self.func
-        return '{0}.{1}'.format(self.func.__module__, self.func.__qualname__)
+        _, func_name = resolve_function_reference(self.func)
+        return func_name
