@@ -100,7 +100,7 @@ class RQScheduler:
         self.log.debug('Acquiring scheduler lock for %s', ', '.join(self._queue_names))
         for name in self._queue_names:
             if self.connection.set(self.get_locking_key(name), pid, nx=True, ex=self.interval + 60):
-                self.log.debug('Acquired scheduler lock for %s', name)
+                self.log.info('Acquired scheduler lock for %s', name)
                 successful_locks.add(name)
 
         # Always reset _scheduled_job_registries when acquiring locks
