@@ -1,10 +1,7 @@
 import json
-import unittest
 from datetime import datetime, timedelta, timezone
 from unittest.mock import patch
 
-from redis import Redis
-
 from rq import Queue, Retry
 from rq.job import Job, JobStatus
 from rq.registry import (
@@ -16,9 +13,8 @@ from rq.registry import (
     StartedJobRegistry,
 )
 from rq.serializers import JSONSerializer
-from rq.utils import get_version
 from rq.worker import Worker
-from tests import RQTestCase
+from tests import RQTestCase, min_redis_version
 from tests.fixtures import echo, say_hello
 
 
@@ -259,7 +255,7 @@ class TestQueue(RQTestCase):
         self.assertEqual(job.origin, barq.name)
         self.assertEqual(job.args[0], 'for Bar', 'Bar should be dequeued second.')
 
-    @unittest.skipIf(get_version(Redis()) < (6, 2, 0), 'Skip if Redis server < 6.2.0')
+    @min_redis_version((6, 2, 0))
     def test_dequeue_any_reliable(self):
         """Dequeueing job from a single queue moves job to intermediate queue."""
         foo_queue = Queue('foo', connection=self.connection)
@@ -282,7 +278,7 @@ class TestQueue(RQTestCase):
         # After job is dequeued, the job ID is in the intermediate queue
         self.assertEqual(self.connection.lpos(foo_queue.intermediate_queue_key, job.id), 1)
 
-    @unittest.skipIf(get_version(Redis()) < (6, 2, 0), 'Skip if Redis server < 6.2.0')
+    @min_redis_version((6, 2, 0))
     def test_intermediate_queue(self):
         """Job should be stuck in intermediate queue if execution fails after dequeued."""
         queue = Queue('foo', connection=self.connection)
@@ -297,7 +293,7 @@ class TestQueue(RQTestCase):
 
             # Job status is still QUEUED even though it's already dequeued
             self.assertEqual(job.get_status(refresh=True), JobStatus.QUEUED)
-            self.assertFalse(job.id in queue.get_job_ids())
+            self.assertNotIn(job.id, queue.get_job_ids())
             self.assertIsNotNone(self.connection.lpos(queue.intermediate_queue_key, job.id))
 
     def test_dequeue_any_ignores_nonexisting_jobs(self):
@@ -442,9 +438,9 @@ class TestQueue(RQTestCase):
         self.assertEqual(len(Queue.all(connection=self.connection)), 3)
 
         # Verify names
-        self.assertTrue('first-queue' in names)
-        self.assertTrue('second-queue' in names)
-        self.assertTrue('third-queue' in names)
+        self.assertIn('first-queue', names)
+        self.assertIn('second-queue', names)
+        self.assertIn('third-queue', names)
 
         # Now empty two queues
         w = Worker([q2, q3], connection=self.connection)
@@ -475,8 +471,8 @@ class TestQueue(RQTestCase):
         self.assertEqual(len(Queue.all(connection=self.connection)), 2)
         names = [q.name for q in Queue.all(connection=self.connection)]
         # Verify names
-        self.assertTrue('queue_with_queued_jobs' in names)
-        self.assertTrue('queue_with_deferred_jobs' in names)
+        self.assertIn('queue_with_queued_jobs', names)
+        self.assertIn('queue_with_deferred_jobs', names)
 
     def test_from_queue_key(self):
         """Ensure being able to get a Queue instance manually from Redis"""
@@ -850,4 +846,4 @@ class TestJobScheduling(RQTestCase):
         job = queue.enqueue_at(scheduled_time, say_hello)
         registry = ScheduledJobRegistry(queue=queue)
         self.assertIn(job, registry)
-        self.assertTrue(registry.get_expiration_time(job), scheduled_time)
+        self.assertEqual(registry.get_expiration_time(job), scheduled_time.replace(microsecond=0))
