@@ -16,7 +16,6 @@ from unittest.mock import Mock
 import psutil
 import pytest
 import redis.exceptions
-from redis import Redis
 
 from rq import Queue, SimpleWorker, Worker
 from rq.defaults import DEFAULT_MAINTENANCE_TASK_INTERVAL, DEFAULT_WORKER_TTL
@@ -25,10 +24,10 @@ from rq.registry import FailedJobRegistry, FinishedJobRegistry, StartedJobRegist
 from rq.results import Result
 from rq.serializers import JSONSerializer
 from rq.suspension import resume, suspend
-from rq.utils import as_text, get_version, now
+from rq.utils import as_text, now
 from rq.version import VERSION
 from rq.worker import HerokuWorker, RandomWorker, RoundRobinWorker, WorkerStatus
-from tests import RQTestCase, find_empty_redis_database, slow
+from tests import RQTestCase, find_empty_redis_database, min_redis_version, slow
 from tests.fixtures import (
     CustomJob,
     access_self,
@@ -94,9 +93,14 @@ class TestWorker(RQTestCase):
         w = Worker([fooq, barq])
         self.assertEqual(w.work(burst=True), False, 'Did not expect any work on the queue.')
 
-        fooq.enqueue(say_hello, name='Frank')
+        job = fooq.enqueue(say_hello, name='Frank')
         self.assertEqual(w.work(burst=True), True, 'Expected at least some work done.')
 
+        # Check that worker_name is stored in the result
+        result = job.latest_result()
+        self.assertIsNotNone(result)
+        self.assertEqual(result.worker_name, w.name)
+
     def test_work_and_quit_custom_serializer(self):
         """Worker processes work, then quits."""
         fooq = Queue('foo', serializer=JSONSerializer, connection=self.connection)
@@ -135,13 +139,13 @@ class TestWorker(RQTestCase):
         self.assertEqual(worker.queues, queues)
         self.assertEqual(worker.get_state(), WorkerStatus.STARTED)
         self.assertEqual(worker._job_id, None)
-        self.assertTrue(worker.key in Worker.all_keys(worker.connection))
+        self.assertIn(worker.key, Worker.all_keys(worker.connection))
         self.assertEqual(worker.version, VERSION)
 
         # If worker is gone, its keys should also be removed
         worker.connection.delete(worker.key)
         Worker.find_by_key(worker.key, connection=self.connection)
-        self.assertFalse(worker.key in Worker.all_keys(worker.connection))
+        self.assertNotIn(worker.key, Worker.all_keys(worker.connection))
 
         self.assertRaises(ValueError, Worker.find_by_key, 'foo', connection=self.connection)
 
@@ -172,9 +176,7 @@ class TestWorker(RQTestCase):
         self.assertEqual(w.work(burst=True), True, 'Expected at least some work done.')
         expected_result = 'Hi there, Frank!'
         self.assertEqual(job.result, expected_result)
-        # Only run if Redis server supports streams
-        if job.supports_redis_streams:
-            self.assertEqual(Result.fetch_latest(job).return_value, expected_result)
+        self.assertEqual(job.latest_result().return_value, expected_result)
         self.assertIsNone(job.worker_name)
 
     def test_job_times(self):
@@ -232,7 +234,7 @@ class TestWorker(RQTestCase):
         self.assertEqual(q.count, 0)
 
         failed_job_registry = FailedJobRegistry(queue=q)
-        self.assertTrue(job in failed_job_registry)
+        self.assertIn(job, failed_job_registry)
 
     def test_meta_is_unserializable(self):
         """Unserializable jobs are put on the failed job registry."""
@@ -249,7 +251,7 @@ class TestWorker(RQTestCase):
         self.connection.hset(job.key, 'meta', invalid_meta)
         job.refresh()
         self.assertIsInstance(job.meta, dict)
-        self.assertTrue('unserialized' in job.meta.keys())
+        self.assertIn('unserialized', job.meta.keys())
 
     @mock.patch('rq.worker.logger.error')
     def test_deserializing_failure_is_handled(self, mock_logger_error):
@@ -281,7 +283,7 @@ class TestWorker(RQTestCase):
         w.perform_job(job, queue)
 
         # An exception should be logged here at ERROR level
-        self.assertIn('Traceback', mock_logger_error.call_args[0][3])
+        self.assertIn('DeserializationError', mock_logger_error.call_args[0][3])
 
     def test_heartbeat(self):
         """Heartbeat saves last_heartbeat"""
@@ -293,7 +295,7 @@ class TestWorker(RQTestCase):
         self.assertEqual(w.hostname, as_text(self.connection.hget(w.key, 'hostname')))
         last_heartbeat = self.connection.hget(w.key, 'last_heartbeat')
         self.assertIsNotNone(self.connection.hget(w.key, 'birth'))
-        self.assertTrue(last_heartbeat is not None)
+        self.assertIsNotNone(last_heartbeat)
         w = Worker.find_by_key(w.key, connection=self.connection)
         self.assertIsInstance(w.last_heartbeat, datetime)
 
@@ -376,7 +378,7 @@ class TestWorker(RQTestCase):
         # Postconditions
         self.assertEqual(q.count, 0)
         failed_job_registry = FailedJobRegistry(queue=q)
-        self.assertTrue(job in failed_job_registry)
+        self.assertIn(job, failed_job_registry)
         self.assertEqual(w.get_current_job_id(), None)
 
         # Check the job
@@ -386,10 +388,10 @@ class TestWorker(RQTestCase):
         # Should be the original enqueued_at date, not the date of enqueueing
         # to the failed queue
         self.assertEqual(job.enqueued_at.replace(tzinfo=timezone.utc).timestamp(), enqueued_at_date.timestamp())
-        if job.supports_redis_streams:
-            result = Result.fetch_latest(job)
-            self.assertTrue(result.exc_string)
-            self.assertEqual(result.type, Result.Type.FAILED)
+        result = job.latest_result()
+        self.assertTrue(result.exc_string)
+        self.assertEqual(result.type, Result.Type.FAILED)
+        self.assertEqual(result.worker_name, w.name)
 
     def test_horse_fails(self):
         """Tests that job status is set to FAILED even if horse unexpectedly fails"""
@@ -410,7 +412,7 @@ class TestWorker(RQTestCase):
         # Postconditions
         self.assertEqual(q.count, 0)
         failed_job_registry = FailedJobRegistry(queue=q)
-        self.assertTrue(job in failed_job_registry)
+        self.assertIn(job, failed_job_registry)
         self.assertEqual(w.get_current_job_id(), None)
 
         # Check the job
@@ -470,7 +472,7 @@ class TestWorker(RQTestCase):
         job.refresh()
         self.assertEqual(job.retries_left, 1)
         self.assertEqual([job.id], queue.job_ids)
-        self.assertFalse(job in registry)
+        self.assertNotIn(job, registry)
 
         # First retry
         queue.empty()
@@ -486,7 +488,12 @@ class TestWorker(RQTestCase):
         self.assertEqual(job.retries_left, 0)
         self.assertEqual([], queue.job_ids)
         # If a job is no longer retries, it's put in FailedJobRegistry
-        self.assertTrue(job in registry)
+        self.assertIn(job, registry)
+
+        # Check that worker_name is stored in the failure result
+        result = job.latest_result()
+        self.assertIsNotNone(result)
+        self.assertEqual(result.worker_name, worker.name)
 
     def test_total_working_time(self):
         """worker.total_working_time is stored properly"""
@@ -526,14 +533,14 @@ class TestWorker(RQTestCase):
         worker.work(burst=True)
 
         registry = FailedJobRegistry(queue=queue)
-        self.assertTrue(job in registry)
+        self.assertIn(job, registry)
 
         # Job is not added to FailedJobRegistry if
         # disable_default_exception_handler is True
         job = queue.enqueue(div_by_zero)
         worker = Worker([queue], disable_default_exception_handler=True)
         worker.work(burst=True)
-        self.assertFalse(job in registry)
+        self.assertNotIn(job, registry)
 
     def test_custom_exc_handling(self):
         """Custom exception handling."""
@@ -828,12 +835,23 @@ class TestWorker(RQTestCase):
         worker = Worker([q], job_class=CustomJob)
         self.assertEqual(worker.job_class, CustomJob)
 
+        # Test job_class as string
+        worker_string = Worker([q], job_class='tests.fixtures.CustomJob')
+        from tests.fixtures import CustomJob as FixturesCustomJob
+
+        self.assertEqual(worker_string.job_class, FixturesCustomJob)
+
     def test_custom_queue_class(self):
         """Ensure Worker accepts custom queue class."""
         q = CustomQueue(connection=self.connection)
         worker = Worker([q], queue_class=CustomQueue)
         self.assertEqual(worker.queue_class, CustomQueue)
 
+        # Test queue_class as string
+        q_generic = Queue(connection=self.connection)
+        worker_string = Worker([q_generic], queue_class='rq.Queue')
+        self.assertEqual(worker_string.queue_class, Queue)
+
     def test_custom_queue_class_is_not_global(self):
         """Ensure Worker custom queue class is not global."""
         q = CustomQueue(connection=self.connection)
@@ -854,6 +872,12 @@ class TestWorker(RQTestCase):
         self.assertEqual(worker_generic.job_class, Job)
         self.assertEqual(Worker.job_class, Job)
 
+        # Test both job_class and queue_class as strings
+        worker = Worker([q], job_class='tests.fixtures.CustomJob')
+        from tests.fixtures import CustomJob as FixturesCustomJob
+
+        self.assertEqual(worker.job_class, FixturesCustomJob)
+
     def test_work_via_simpleworker(self):
         """Worker processes work, with forking disabled,
         then returns."""
@@ -886,8 +910,11 @@ class TestWorker(RQTestCase):
 
         # Updates working queue, job execution should be there
         registry = StartedJobRegistry(connection=self.connection)
-        self.assertTrue(job.id in registry.get_job_ids())
-        self.assertTrue((worker.execution.job_id, worker.execution.id) in registry.get_job_and_execution_ids())
+        self.assertIn(job.id, registry.get_job_ids())
+        self.assertIn(
+            (worker.execution.job_id, worker.execution.id),
+            registry.get_job_and_execution_ids(),
+        )
 
         # Updates worker's current job
         self.assertEqual(worker.get_current_job_id(), job.id)
@@ -909,7 +936,7 @@ class TestWorker(RQTestCase):
         self.assertEqual(worker.get_current_job_id(), None)
         self.assertIsNone(worker.execution)
 
-    @skipIf(get_version(Redis()) < (6, 2, 0), 'Skip if Redis server < 6.2.0')
+    @min_redis_version((6, 2, 0))
     def test_prepare_job_execution_removes_key_from_intermediate_queue(self):
         """Prepare job execution removes job from intermediate queue."""
         queue = Queue(connection=self.connection)
@@ -922,7 +949,7 @@ class TestWorker(RQTestCase):
         self.assertIsNone(self.connection.lpos(queue.intermediate_queue_key, job.id))
         self.assertEqual(queue.count, 0)
 
-    @skipIf(get_version(Redis()) < (6, 2, 0), 'Skip if Redis server < 6.2.0')
+    @min_redis_version((6, 2, 0))
     def test_work_removes_key_from_intermediate_queue(self):
         """Worker removes job from intermediate queue."""
         queue = Queue(connection=self.connection)
@@ -1161,7 +1188,7 @@ class TestWorker(RQTestCase):
         # Postconditions
         self.assertEqual(q.count, 0)
         failed_job_registry = FailedJobRegistry(queue=q)
-        self.assertTrue(job in failed_job_registry)
+        self.assertIn(job, failed_job_registry)
         self.assertEqual(w.get_current_job_id(), None)
 
         job_check = Job.fetch(job.id, connection=self.connection)
@@ -1359,6 +1386,21 @@ class TestWorker(RQTestCase):
         q.enqueue(say_hello, args=('ccc',), result_ttl=0)
         self.assertTrue(w.work(burst=True))
 
+    def test_custom_job_and_queue(self):
+        class CustomJob(Job):
+            pass
+
+        class CustomQueue(Queue):
+            pass
+
+        class CustomWorker(Worker):
+            job_class = CustomJob
+            queue_class = CustomQueue
+
+        worker = CustomWorker(CustomQueue('foo', connection=self.connection))
+        assert worker.job_class is CustomJob
+        assert worker.queue_class is CustomQueue
+
 
 def wait_and_kill_work_horse(pid, time_to_wait=0.0):
     time.sleep(time_to_wait)
@@ -1423,7 +1465,7 @@ class WorkerShutdownTestCase(TimeoutTestCase, RQTestCase):
 
         sentinel_file = '/tmp/.rq_sentinel_cold'
         self.assertFalse(
-            os.path.exists(sentinel_file), '{sentinel_file} file should not exist yet, delete that file and try again.'
+            os.path.exists(sentinel_file), f'{sentinel_file} file should not exist yet, delete that file and try again.'
         )
         fooq.enqueue(create_file_after_timeout, sentinel_file, 5)
         self.assertFalse(w._stop_requested)
@@ -1442,7 +1484,7 @@ class WorkerShutdownTestCase(TimeoutTestCase, RQTestCase):
 
     @slow
     def test_work_horse_death_sets_job_failed(self):
-        """worker with an ongoing job whose work horse dies unexpectadly (before
+        """worker with an ongoing job whose work horse dies unexpectedly (before
         completing the job) should set the job's status to FAILED
         """
         fooq = Queue('foo', connection=self.connection)
@@ -1461,7 +1503,7 @@ class WorkerShutdownTestCase(TimeoutTestCase, RQTestCase):
         p.join(1)
         self.assertEqual(job_status, JobStatus.FAILED)
         failed_job_registry = FailedJobRegistry(queue=fooq)
-        self.assertTrue(job in failed_job_registry)
+        self.assertIn(job, failed_job_registry)
         self.assertEqual(fooq.count, 0)
 
     @slow
@@ -1497,10 +1539,10 @@ class WorkerShutdownTestCase(TimeoutTestCase, RQTestCase):
         total_time = w.job_monitoring_interval + 65 + fudge_factor
 
         right_now = now()
-        self.assertTrue((now() - right_now).total_seconds() < total_time)
+        self.assertLess((now() - right_now).total_seconds(), total_time)
         self.assertEqual(job.get_status(), JobStatus.FAILED)
         failed_job_registry = FailedJobRegistry(queue=fooq)
-        self.assertTrue(job in failed_job_registry)
+        self.assertIn(job, failed_job_registry)
         self.assertEqual(fooq.count, 0)
         self.assertFalse(psutil.pid_exists(subprocess_pid))
 
@@ -1527,7 +1569,7 @@ class TestWorkerSubprocess(RQTestCase):
         job = q.enqueue(access_self)
         subprocess.check_call(['rqworker', '-u', self.redis_url, '-b'])
         registry = FinishedJobRegistry(queue=q)
-        self.assertTrue(job in registry)
+        self.assertIn(job, registry)
         assert q.count == 0
 
     @skipIf('pypy' in sys.version.lower(), 'often times out with pypy')
@@ -1537,7 +1579,7 @@ class TestWorkerSubprocess(RQTestCase):
         job = q.enqueue(schedule_access_self)
         subprocess.check_call(['rqworker', '-u', self.redis_url, '-b'])
         registry = FinishedJobRegistry(queue=q)
-        self.assertTrue(job in registry)
+        self.assertIn(job, registry)
         assert q.count == 0
 
 
@@ -1600,7 +1642,7 @@ class HerokuWorkerShutdownTestCase(TimeoutTestCase, RQTestCase):
         self.assertFalse(os.path.exists(os.path.join(self.sandbox, 'finished')))
 
     @mock.patch('rq.worker.logger.info')
-    def test_handle_shutdown_request(self, mock_logger_info):
+    def test_handle_shutdown_request(self, mock_logger):
         """Mutate HerokuWorker so _horse_pid refers to an artificial process
         and test handle_warm_shutdown_request"""
         w = HerokuWorker('foo', connection=self.connection)
@@ -1617,7 +1659,7 @@ class HerokuWorkerShutdownTestCase(TimeoutTestCase, RQTestCase):
         # would expect p.exitcode to be -34
         self.assertEqual(p.exitcode, -34)
         self.assertFalse(os.path.exists(path))
-        mock_logger_info.assert_called_with('Killed horse pid %s', p.pid)
+        mock_logger.assert_called_with('Worker %s: killed horse pid %s', w.name, p.pid)
 
     def test_handle_shutdown_request_no_horse(self):
         """Mutate HerokuWorker so _horse_pid refers to non existent process
